@@ -4,26 +4,17 @@
  * Functions required for dlopen et. al.
  */
 
-#include <stdlib.h>
-#include <features.h>
-#include "dlfcn.h"
-#include "linuxelf.h"
-#include "ld_syscall.h"
-#include "ld_hash.h"
-#include "ld_string.h"
+#include <ldso.h>
 
-extern struct r_debug *_dl_debug_addr;
-
-extern void *(*_dl_malloc_function) (size_t size);
-
-static int do_fixup(struct elf_resolve *tpnt, int flag);
-static int do_dlclose(void *, int need_fini);
 
+/* The public interfaces */
 void *dlopen(const char *, int) __attribute__ ((__weak__, __alias__ ("_dlopen")));
-const char *dlerror(void) __attribute__ ((__weak__, __alias__ ("_dlerror")));
-void *dlsym(void *, const char *) __attribute__ ((__weak__, __alias__ ("_dlsym")));
 int dlclose(void *) __attribute__ ((__weak__, __alias__ ("_dlclose")));
+void *dlsym(void *, const char *) __attribute__ ((__weak__, __alias__ ("_dlsym")));
+const char *dlerror(void) __attribute__ ((__weak__, __alias__ ("_dlerror")));
 int dladdr(void *, Dl_info *) __attribute__ ((__weak__, __alias__ ("_dladdr")));
+void _dlinfo(void);
+
 
 #ifdef __PIC__
 /* This is a real hack.  We need access to the dynamic linker, but we
@@ -43,10 +34,12 @@ extern char *_dl_find_hash(const char *, struct dyn_elf *, struct elf_resolve *,
 	__attribute__ ((__weak__, __alias__ ("foobar")));
 extern struct elf_resolve * _dl_load_shared_library(int, struct dyn_elf **, struct elf_resolve *, char *)
 	__attribute__ ((__weak__, __alias__ ("foobar")));
-extern int _dl_parse_relocation_information(struct elf_resolve *, unsigned long, unsigned long, int)
-	__attribute__ ((__weak__, __alias__ ("foobar")));
-extern void _dl_parse_lazy_relocation_information(struct elf_resolve *, unsigned long, unsigned long, int)
+extern struct elf_resolve * _dl_check_if_named_library_is_loaded(const char *full_libname)
 	__attribute__ ((__weak__, __alias__ ("foobar")));
+extern int _dl_fixup(struct elf_resolve *tpnt, int lazy)
+	 __attribute__ ((__weak__, __alias__ ("foobar")));
+extern int _dl_copy_fixups(struct dyn_elf * tpnt)
+	 __attribute__ ((__weak__, __alias__ ("foobar")));
 #ifdef __mips__
 extern void _dl_perform_mips_global_got_relocations(struct elf_resolve *tpnt)
 	__attribute__ ((__weak__, __alias__ ("foobar")));
@@ -62,28 +55,44 @@ extern struct elf_resolve *_dl_loaded_modules __attribute__ ((__weak__, __alias_
 extern struct r_debug *_dl_debug_addr __attribute__ ((__weak__, __alias__ ("foobar1")));
 extern unsigned long _dl_error_number __attribute__ ((__weak__, __alias__ ("foobar1")));
 extern void *(*_dl_malloc_function)(size_t) __attribute__ ((__weak__, __alias__ ("foobar1")));
-#else
 #ifdef __SUPPORT_LD_DEBUG__
-static char *_dl_debug  = 0;
-static char *_dl_debug_symbols = 0;
-static char *_dl_debug_move    = 0;
-static char *_dl_debug_reloc   = 0;
-static char *_dl_debug_detail  = 0;
-static char *_dl_debug_nofixups  = 0;
-static char *_dl_debug_bindings  = 0;
-static int   _dl_debug_file = 2;
-#elif defined __SUPPORT_LD_DEBUG_EARLY__
-#define _dl_debug_file 2
+extern char *_dl_debug __attribute__ ((__weak__, __alias__ ("foobar1")));
+extern char *_dl_debug_symbols __attribute__ ((__weak__, __alias__ ("foobar1")));
+extern char *_dl_debug_move __attribute__ ((__weak__, __alias__ ("foobar1")));
+extern char *_dl_debug_reloc __attribute__ ((__weak__, __alias__ ("foobar1")));
+extern char *_dl_debug_detail __attribute__ ((__weak__, __alias__ ("foobar1")));
+extern char *_dl_debug_nofixups __attribute__ ((__weak__, __alias__ ("foobar1")));
+extern char *_dl_debug_bindings __attribute__ ((__weak__, __alias__ ("foobar1")));
+extern int   _dl_debug_file __attribute__ ((__weak__, __alias__ ("foobar1")));
+#endif
+
+#else	/* __PIC__ */
+
+#ifdef __SUPPORT_LD_DEBUG__
+char *_dl_debug  = 0;
+char *_dl_debug_symbols = 0;
+char *_dl_debug_move    = 0;
+char *_dl_debug_reloc   = 0;
+char *_dl_debug_detail  = 0;
+char *_dl_debug_nofixups  = 0;
+char *_dl_debug_bindings  = 0;
+int   _dl_debug_file = 2;
 #endif
 char *_dl_library_path = 0;
 char *_dl_ldsopath = 0;
 struct r_debug *_dl_debug_addr = NULL;
 static char *_dl_malloc_addr, *_dl_mmap_zero;
-#include "../ldso/ldso.h"               /* Pull in the name of ld.so */
+#include "../ldso/_dl_progname.h"               /* Pull in the name of ld.so */
 #include "../ldso/hash.c"
+#define _dl_trace_loaded_objects    0
 #include "../ldso/readelflib1.c"
+void *(*_dl_malloc_function) (size_t size);
+int _dl_fixup(struct elf_resolve *tpnt, int lazy);
 #endif
 
+static int do_dlclose(void *, int need_fini);
+
+
 static const char *dl_error_names[] = {
 	"",
 	"File not found",
@@ -123,16 +132,13 @@ static void __attribute__ ((destructor)) dl_cleanup(void)
 
 void *_dlopen(const char *libname, int flag)
 {
-	struct elf_resolve *tpnt, *tfrom;
-	struct dyn_elf *rpnt = NULL;
-	struct dyn_elf *dyn_chain;
+	struct elf_resolve *tpnt, *tfrom, *tcurr;
+	struct dyn_elf *dyn_chain, *rpnt = NULL;
 	struct dyn_elf *dpnt;
 	static int dl_init = 0;
-	char *from;
+	ElfW(Addr) from;
+	struct elf_resolve *tpnt1;
 	void (*dl_brk) (void);
-#ifdef __PIC__
-	int (*dl_elf_init) (void);
-#endif
 
 	/* A bit of sanity checking... */
 	if (!(flag & (RTLD_LAZY|RTLD_NOW))) {
@@ -140,7 +146,7 @@ void *_dlopen(const char *libname, int flag)
 		return NULL;
 	}
 
-	from = __builtin_return_address(0);
+	from = (ElfW(Addr)) __builtin_return_address(0);
 
 	/* Have the dynamic linker use the regular malloc function now */
 	if (!dl_init) {
@@ -152,9 +158,7 @@ void *_dlopen(const char *libname, int flag)
 	if (!libname)
 		return _dl_symbol_tables;
 
-#ifdef USE_CACHE
 	_dl_map_cache();
-#endif
 
 	/*
 	 * Try and locate the module we were called from - we
@@ -168,94 +172,145 @@ void *_dlopen(const char *libname, int flag)
 	for (dpnt = _dl_symbol_tables; dpnt; dpnt = dpnt->next) {
 		tpnt = dpnt->dyn;
 		if (tpnt->loadaddr < from
-			&& (tfrom == NULL || tfrom->loadaddr < tpnt->loadaddr))
+				&& (tfrom == NULL || tfrom->loadaddr < tpnt->loadaddr))
 			tfrom = tpnt;
 	}
 
-	if (!(tpnt = _dl_load_shared_library(0, &rpnt, tfrom, (char*)libname))) {
-#ifdef USE_CACHE
-		_dl_unmap_cache();
+	/* Try to load the specified library */
+#ifdef __SUPPORT_LD_DEBUG__
+	if(_dl_debug) 
+	_dl_dprintf(_dl_debug_file, "Trying to dlopen '%s'\n", (char*)libname);
 #endif
+	if (!(tpnt = _dl_load_shared_library(0, &rpnt, tfrom, (char*)libname))) {
+		_dl_unmap_cache();
 		return NULL;
 	}
 	//tpnt->libtype = loaded_file;
 
-	dyn_chain = rpnt = (struct dyn_elf *) malloc(sizeof(struct dyn_elf));
-	_dl_memset(rpnt, 0, sizeof(*rpnt));
-	rpnt->dyn = tpnt;
-	rpnt->flags = flag;
+
+	dyn_chain = (struct dyn_elf *) malloc(sizeof(struct dyn_elf));
+	_dl_memset(dyn_chain, 0, sizeof(struct dyn_elf));
+	dyn_chain->dyn = tpnt;
+	dyn_chain->flags = flag;
 	if (!tpnt->symbol_scope)
 		tpnt->symbol_scope = dyn_chain;
 
-	rpnt->next_handle = _dl_handles;
-	_dl_handles = rpnt;
+	dyn_chain->next_handle = _dl_handles;
+	_dl_handles = rpnt = dyn_chain;
 
-	/*
-	 * OK, we have the requested file in memory.  Now check for
-	 * any other requested files that may also be required.
-	 */
-	  {
-	    struct elf_resolve *tcurr;
-	    struct elf_resolve * tpnt1;
-	    Elf32_Dyn * dpnt;
-	    char * lpnt;
 
-	    tcurr = tpnt;
-	    do{
-	      for(dpnt = (Elf32_Dyn *) tcurr->dynamic_addr; dpnt->d_tag; dpnt++)
-		{
-	  
-		  if(dpnt->d_tag == DT_NEEDED)
-		    {
-		      lpnt = tcurr->loadaddr + tcurr->dynamic_info[DT_STRTAB] + 
-			dpnt->d_un.d_val;
-		      if(!(tpnt1 = _dl_load_shared_library(0, &rpnt, tcurr, lpnt)))
-			goto oops;
+#ifdef __SUPPORT_LD_DEBUG__
+	if(_dl_debug) 
+	_dl_dprintf(_dl_debug_file, "Looking for needed libraries\n");
+#endif
+	for (tcurr = tpnt; tcurr; tcurr = tcurr->next)
+	{
+		Elf32_Dyn *dpnt;
+		char *lpntstr;
+		for (dpnt = (Elf32_Dyn *) tcurr->dynamic_addr; dpnt->d_tag; dpnt++) {
+			if (dpnt->d_tag == DT_NEEDED) {
 
-		      rpnt->next = (struct dyn_elf *) malloc(sizeof(struct dyn_elf));
-		      _dl_memset (rpnt->next, 0, sizeof (*(rpnt->next)));
-		      rpnt = rpnt->next;
-		      if (!tpnt1->symbol_scope) tpnt1->symbol_scope = dyn_chain;
-		      rpnt->dyn = tpnt1;
-		    };
+				char *name;
+				lpntstr = (char*) (tcurr->loadaddr + tcurr->dynamic_info[DT_STRTAB] + 
+					dpnt->d_un.d_val);
+				name = _dl_get_last_path_component(lpntstr);
+
+#ifdef __SUPPORT_LD_DEBUG__
+				if(_dl_debug) 
+				_dl_dprintf(_dl_debug_file, "Trying to load '%s', needed by '%s'\n", 
+						lpntstr, tcurr->libname);
+#endif
+#if 0
+				if ((tpnt1 = _dl_check_if_named_library_is_loaded(name)))
+				{
+				    /* If the needed library is already loaded, then we
+				     * need to override all globals and weaks in the current
+				     * chain...  Or something. */
+				    continue;
+				}
+#endif
+
+				if (!(tpnt1 = _dl_load_shared_library(0, &rpnt, tcurr, lpntstr))) {
+					goto oops;
+				}
+				/* We need global symbol resolution for everything
+				 * in the dependent chain */
+				dyn_chain->flags |= RTLD_GLOBAL;
+
+				rpnt->next = (struct dyn_elf *) malloc(sizeof(struct dyn_elf));
+				_dl_memset (rpnt->next, 0, sizeof (struct dyn_elf));
+				rpnt = rpnt->next;
+				if (!tpnt1->symbol_scope) tpnt1->symbol_scope = rpnt;
+				rpnt->dyn = tpnt1;
+
+			}
 		}
-	      
-	      tcurr = tcurr->next;
-	    } while(tcurr);
-	  }
-	 
+	}
+
 	/*
 	 * OK, now attach the entire chain at the end
 	 */
-
 	rpnt->next = _dl_symbol_tables;
 
-	/*
-	 * MIPS is special *sigh*
-	 */
 #ifdef __mips__
+	/*
+	 * Relocation of the GOT entries for MIPS have to be done
+	 * after all the libraries have been loaded.
+	 */
 	_dl_perform_mips_global_got_relocations(tpnt);
 #endif
 
-	if (do_fixup(tpnt, flag)) {
-		_dl_error_number = LD_NO_SYMBOL;
+#ifdef __SUPPORT_LD_DEBUG__
+	if(_dl_debug) 
+	_dl_dprintf(_dl_debug_file, "Beginning dlopen relocation fixups\n");
+#endif
+	/*
+	 * OK, now all of the kids are tucked into bed in their proper addresses.
+	 * Now we go through and look for REL and RELA records that indicate fixups
+	 * to the GOT tables.  We need to do this in reverse order so that COPY
+	 * directives work correctly */
+	if (_dl_fixup(dyn_chain->dyn, (dyn_chain->flags & RTLD_LAZY)))
 		goto oops;
+
+#ifdef __SUPPORT_LD_DEBUG__
+	if(_dl_debug) 
+	_dl_dprintf(_dl_debug_file, "Beginning dlopen copy fixups\n");
+#endif
+	if (_dl_symbol_tables) {
+		if (_dl_copy_fixups(dyn_chain))
+			goto oops;
 	}
 
+
+	/* TODO:  Should we set the protections of all pages back to R/O now ? */
+	
+
+	/* Notify the debugger we have added some objects. */
+	_dl_debug_addr->r_state = RT_ADD;
 	if (_dl_debug_addr) {
-	    dl_brk = (void (*)(void)) _dl_debug_addr->r_brk;
-	    if (dl_brk != NULL) {
-		_dl_debug_addr->r_state = RT_ADD;
-		(*dl_brk) ();
+		dl_brk = (void (*)(void)) _dl_debug_addr->r_brk;
+		if (dl_brk != NULL) {
+			_dl_debug_addr->r_state = RT_ADD;
+			(*dl_brk) ();
 
-		_dl_debug_addr->r_state = RT_CONSISTENT;
-		(*dl_brk) ();
-	    }
+			_dl_debug_addr->r_state = RT_CONSISTENT;
+			(*dl_brk) ();
+		}
 	}
 
+#if 0 //def __SUPPORT_LD_DEBUG__
+	if(_dl_debug) 
+	_dlinfo();
+#endif
+
 #ifdef __PIC__
-	for (rpnt = dyn_chain; rpnt; rpnt = rpnt->next) {
-		tpnt = rpnt->dyn;
+	/* Find the last library so we can run things in the right order */
+	for (tpnt = dyn_chain->dyn; tpnt->next!=NULL; tpnt = tpnt->next)
+	    ;
+
+	/* Run the ctors and set up the dtors */
+	for (; tpnt != dyn_chain->dyn->prev; tpnt=tpnt->prev)
+	{
 		/* Apparently crt1 for the application is responsible for handling this.
 		 * We only need to run the init/fini for shared libraries
 		 */
@@ -268,84 +323,43 @@ void *_dlopen(const char *libname, int flag)
 		tpnt->init_flag |= INIT_FUNCS_CALLED;
 
 		if (tpnt->dynamic_info[DT_INIT]) {
-			dl_elf_init = (int (*)(void)) (tpnt->loadaddr + tpnt->dynamic_info[DT_INIT]);
-			(*dl_elf_init) ();
+		    void (*dl_elf_func) (void);
+		    dl_elf_func = (void (*)(void)) (tpnt->loadaddr + tpnt->dynamic_info[DT_INIT]);
+		    if (dl_elf_func && *dl_elf_func != NULL) {
+#ifdef __SUPPORT_LD_DEBUG__
+			if(_dl_debug) 
+			_dl_dprintf(2, "running ctors for library %s at '%x'\n", tpnt->libname, dl_elf_func);
+#endif
+			(*dl_elf_func) ();
+		    }
 		}
 		if (tpnt->dynamic_info[DT_FINI]) {
-			atexit((void (*)(void)) (tpnt->loadaddr + tpnt->dynamic_info[DT_FINI]));
-		}
-
-	}
+		    void (*dl_elf_func) (void);
+		    dl_elf_func = (void (*)(void)) (tpnt->loadaddr + tpnt->dynamic_info[DT_FINI]);
+		    if (dl_elf_func && *dl_elf_func != NULL) {
+#ifdef __SUPPORT_LD_DEBUG__
+			if(_dl_debug) 
+			_dl_dprintf(2, "setting up dtors for library %s at '%x'\n", tpnt->libname, dl_elf_func);
 #endif
-
-#ifdef USE_CACHE
-	_dl_unmap_cache();
+			atexit(dl_elf_func);
+		    }
+		}
+	}
 #endif
 	return (void *) dyn_chain;
 
-  oops:
+oops:
 	/* Something went wrong.  Clean up and return NULL. */
-#ifdef USE_CACHE
 	_dl_unmap_cache();
-#endif
 	do_dlclose(dyn_chain, 0);
 	return NULL;
 }
 
-static int do_fixup(struct elf_resolve *tpnt, int flag)
-{
-	int goof = 0;
-
-	if (tpnt->next)
-		goof += do_fixup(tpnt->next, flag);
-
-	if (tpnt->dynamic_info[DT_REL]) {
-#ifdef ELF_USES_RELOCA
-		goof++;
-#else
-		if (tpnt->init_flag & RELOCS_DONE)
-			return goof;
-		tpnt->init_flag |= RELOCS_DONE;
-
-		goof += _dl_parse_relocation_information(tpnt, 
-			tpnt->dynamic_info[DT_REL], tpnt->dynamic_info[DT_RELSZ], 0);
-#endif
-	}
-	if (tpnt->dynamic_info[DT_RELA]) {
-#ifdef ELF_USES_RELOCA
-		if (tpnt->init_flag & RELOCS_DONE)
-			return goof;
-		tpnt->init_flag |= RELOCS_DONE;
-
-		goof += _dl_parse_relocation_information(tpnt, 
-			tpnt->dynamic_info[DT_RELA], tpnt->dynamic_info[DT_RELASZ], 0);
-#else
-		goof++;
-#endif
-	}
-	if (tpnt->dynamic_info[DT_JMPREL]) {
-		if (tpnt->init_flag & JMP_RELOCS_DONE)
-			return goof;
-		tpnt->init_flag |= JMP_RELOCS_DONE;
-
-		if (flag == RTLD_LAZY) {
-			_dl_parse_lazy_relocation_information(tpnt, 
-				tpnt->dynamic_info[DT_JMPREL], 
-				tpnt->dynamic_info[DT_PLTRELSZ], 0);
-		} else {
-			goof += _dl_parse_relocation_information(tpnt, 
-				tpnt->dynamic_info[DT_JMPREL], 
-				tpnt->dynamic_info[DT_PLTRELSZ], 0);
-		}
-	};
-	return goof;
-}
-
 void *_dlsym(void *vhandle, const char *name)
 {
 	struct elf_resolve *tpnt, *tfrom;
 	struct dyn_elf *handle;
-	char *from;
+	ElfW(Addr) from;
 	struct dyn_elf *rpnt;
 	void *ret;
 
@@ -372,7 +386,7 @@ void *_dlsym(void *vhandle, const char *name)
 		 * dynamic loader itself, as it doesn't know
 		 * how to properly treat it.
 		 */
-		from = __builtin_return_address(0);
+		from = (ElfW(Addr)) __builtin_return_address(0);
 
 		tfrom = NULL;
 		for (rpnt = _dl_symbol_tables; rpnt; rpnt = rpnt->next) {
@@ -404,7 +418,7 @@ static int do_dlclose(void *vhandle, int need_fini)
 {
 	struct dyn_elf *rpnt, *rpnt1;
 	struct dyn_elf *spnt, *spnt1;
-	elf_phdr *ppnt;
+	ElfW(Phdr) *ppnt;
 	struct elf_resolve *tpnt;
 	int (*dl_elf_fini) (void);
 	void (*dl_brk) (void);
@@ -489,7 +503,7 @@ static int do_dlclose(void *vhandle, int need_fini)
 				if (end < ppnt->p_vaddr + ppnt->p_memsz)
 					end = ppnt->p_vaddr + ppnt->p_memsz;
 			}
-			_dl_munmap(rpnt->dyn->loadaddr, end);
+			_dl_munmap((void*)rpnt->dyn->loadaddr, end);
 			/* Next, remove rpnt->dyn from the loaded_module list */
 			if (_dl_loaded_modules == rpnt->dyn) {
 				_dl_loaded_modules = rpnt->dyn->next;
@@ -574,9 +588,7 @@ int _dladdr(void *__address, Dl_info * __dlip)
 	struct elf_resolve *pelf;
 	struct elf_resolve *rpnt;
 
-#ifdef USE_CACHE
 	_dl_map_cache();
-#endif
 
 	/*
 	 * Try and locate the module address is in
@@ -584,7 +596,7 @@ int _dladdr(void *__address, Dl_info * __dlip)
 	pelf = NULL;
 
 #if 0
-	_dl_dprintf(2, "dladdr( 0x%p, 0x%p )\n", __address, __dlip);
+	_dl_dprintf(2, "dladdr( %x, %x )\n", __address, __dlip);
 #endif
 
 	for (rpnt = _dl_loaded_modules; rpnt; rpnt = rpnt->next) {
@@ -592,10 +604,10 @@ int _dladdr(void *__address, Dl_info * __dlip)
 
 		tpnt = rpnt;
 #if 0
-		_dl_dprintf(2, "Module \"%s\" at 0x%p\n", 
+		_dl_dprintf(2, "Module \"%s\" at %x\n", 
 			tpnt->libname, tpnt->loadaddr);
 #endif
-		if (tpnt->loadaddr < (char *) __address
+		if (tpnt->loadaddr < (ElfW(Addr)) __address
 			&& (pelf == NULL || pelf->loadaddr < tpnt->loadaddr)) {
 		    pelf = tpnt;
 		}
@@ -615,24 +627,25 @@ int _dladdr(void *__address, Dl_info * __dlip)
 		int hn, si;
 		int sf;
 		int sn = 0;
-		void *sa = 0;
+		ElfW(Addr) sa;
 
+		sa = 0;
 		symtab = (Elf32_Sym *) (pelf->dynamic_info[DT_SYMTAB] + pelf->loadaddr);
 		strtab = (char *) (pelf->dynamic_info[DT_STRTAB] + pelf->loadaddr);
 
 		sf = 0;
 		for (hn = 0; hn < pelf->nbucket; hn++) {
 			for (si = pelf->elf_buckets[hn]; si; si = pelf->chains[si]) {
-				void *symbol_addr;
+				ElfW(Addr) symbol_addr;
 
 				symbol_addr = pelf->loadaddr + symtab[si].st_value;
-				if (symbol_addr <= __address && (!sf || sa < symbol_addr)) {
+				if (symbol_addr <= (ElfW(Addr))__address && (!sf || sa < symbol_addr)) {
 					sa = symbol_addr;
 					sn = si;
 					sf = 1;
 				}
 #if 0
-				_dl_dprintf(2, "Symbol \"%s\" at 0x%p\n", 
+				_dl_dprintf(2, "Symbol \"%s\" at %x\n", 
 					strtab + symtab[si].st_name, symbol_addr);
 #endif
 			}
@@ -640,9 +653,9 @@ int _dladdr(void *__address, Dl_info * __dlip)
 
 		if (sf) {
 			__dlip->dli_fname = pelf->libname;
-			__dlip->dli_fbase = pelf->loadaddr;
+			__dlip->dli_fbase = (void *)pelf->loadaddr;
 			__dlip->dli_sname = strtab + symtab[sn].st_name;
-			__dlip->dli_saddr = sa;
+			__dlip->dli_saddr = (void *)sa;
 		}
 		return 1;
 	}
