@@ -2,8 +2,7 @@
 /*
  * Syscalls for uClibc
  *
- * Copyright (C) 2000 by Lineo, inc
- * Copyright (C) 2001, 2002 by Erik Andersen
+ * Copyright (C) 2001-2003 by Erik Andersen
  * Written by Erik Andersen <andersen@codpoet.org>
  *
  * This program is free software; you can redistribute it and/or modify it
@@ -47,14 +46,14 @@
 #	ifdef __UCLIBC_HAS_MMU__
 #define __NR___libc_fork __NR_fork
 		_syscall0(pid_t, __libc_fork);
-		weak_alias (__libc_fork, fork)
 #	else
-		pid_t fork(void)
+		pid_t __libc_fork(void)
 		{
 			__set_errno(ENOSYS);
 			return -1;
 		}
 #	endif
+weak_alias (__libc_fork, fork)
 #endif
 
 //#define __NR_read             3
@@ -272,19 +271,25 @@ int setuid(uid_t uid)
 #endif
 
 //#define __NR_getuid           24
-#ifdef L_getuid
+#ifdef L___syscall_getuid
 #include <unistd.h>
 #if defined (__alpha__)
 #define __NR_getuid     __NR_getxuid
 #endif
-_syscall0(uid_t, getuid);
+#define __NR___syscall_getuid __NR_getuid
+static inline 
+_syscall0(int, __syscall_getuid);
+uid_t getuid(void)
+{
+	return(__syscall_getuid());
+}
 #endif
 
 //#define __NR_stime            25
 #ifdef L_stime
 #include <time.h>
 #include <sys/time.h>
-#ifdef _NR_stime
+#ifdef __NR_stime
 _syscall1(int, stime, const time_t *, t);
 #else
 int stime(const time_t *when)
@@ -521,21 +526,33 @@ int setgid(gid_t gid)
 #endif
 
 //#define __NR_getgid           47
-#ifdef L_getgid
+#ifdef L___syscall_getgid
 #include <unistd.h>
+#define __NR___syscall_getgid __NR_getgid
 #if defined (__alpha__)
 #define __NR_getgid     __NR_getxgid
 #endif
-_syscall0(gid_t, getgid);
+static inline 
+_syscall0(int, __syscall_getgid);
+gid_t getgid(void)
+{
+	return(__syscall_getgid());
+}
 #endif
 
 //#define __NR_signal           48
 
 //#define __NR_geteuid          49
-#ifdef	L_geteuid
+#ifdef	L___syscall_geteuid
 #include <unistd.h>
 #	ifdef	__NR_geteuid
-	_syscall0(uid_t, geteuid);
+#define __NR___syscall_geteuid __NR_geteuid
+	static inline 
+	_syscall0(int, __syscall_geteuid);
+	uid_t geteuid(void)
+	{
+		return(__syscall_geteuid());
+	}
 #	else
 	uid_t geteuid(void)
 	{
@@ -545,10 +562,16 @@ _syscall0(gid_t, getgid);
 #endif
 
 //#define __NR_getegid          50
-#ifdef	L_getegid
+#ifdef	L___syscall_getegid
 #include <unistd.h>
 #	ifdef	__NR_getegid
-	_syscall0(gid_t, getegid);
+#define __NR___syscall_getegid __NR_getegid
+static inline 
+_syscall0(int, __syscall_getegid);
+gid_t getegid(void)
+{
+	return(__syscall_getegid());
+}
 #	else
 	gid_t getegid(void)
 	{
@@ -593,24 +616,27 @@ _syscall3(int, __syscall_ioctl, int, fd, int, request, void *, arg);
 #endif
 
 //#define __NR_fcntl            55
-#ifdef L__fcntl
+#ifdef L___syscall_fcntl
 #include <stdarg.h>
 #include <fcntl.h>
-#define __NR__fcntl __NR_fcntl
-extern int _fcntl(int fd, int cmd, long arg);
-
-_syscall3(int, _fcntl, int, fd, int, cmd, long, arg);
-
-int __libc_fcntl(int fd, int command, ...)
+#define __NR___syscall_fcntl __NR_fcntl
+#ifdef __UCLIBC_HAS_LFS__
+static inline
+#endif
+_syscall3(int, __syscall_fcntl, int, fd, int, cmd, long, arg);
+int __libc_fcntl(int fd, int cmd, ...)
 {
 	long arg;
 	va_list list;
-
-	va_start(list, command);
+	if (cmd == F_GETLK64 || cmd == F_SETLK64 || cmd == F_SETLKW64) 
+	{
+			__set_errno(ENOSYS);
+			return -1;
+	}
+	va_start(list, cmd);
 	arg = va_arg(list, long);
-
 	va_end(list);
-	return _fcntl(fd, command, arg);
+	return(__syscall_fcntl(fd, cmd, arg));
 }
 weak_alias(__libc_fcntl, fcntl)
 #endif
@@ -630,6 +656,7 @@ int setpgid(pid_t pid, pid_t pgid)
 #endif
 
 //#define __NR_ulimit           58
+//See ulimit.c
 
 //#define __NR_oldolduname      59
 
@@ -652,6 +679,20 @@ _syscall1(int, chroot, const char *, path);
 #endif
 
 //#define __NR_ustat            62
+#ifdef L___syscall_ustat
+#define __NR___syscall_ustat __NR_ustat
+#include <sys/ustat.h>
+static inline
+_syscall2(int, __syscall_ustat, unsigned short int, kdev_t, struct ustat *, ubuf);
+int ustat(dev_t dev, struct ustat *ubuf)
+{ 
+	__kernel_dev_t k_dev;
+	/* We must convert the dev_t value to a __kernel_dev_t */
+	k_dev = ((major(dev) & 0xff) << 8) | (minor(dev) & 0xff);
+	return __syscall_ustat(k_dev, ubuf);
+}
+#endif
+
 
 //#define __NR_dup2             63
 #ifdef L_dup2
@@ -848,7 +889,7 @@ int getgroups(int n, gid_t *groups)
 		ngids = __syscall_getgroups(n, kernel_groups);
 		if (n != 0 && ngids > 0) {
 			for (i = 0; i < ngids; i++) {
-				(groups)[i] = kernel_groups[i];
+				groups[i] = kernel_groups[i];
 			}
 		}
 		return ngids;
@@ -1345,6 +1386,12 @@ _syscall2(int, bdflush, int, __func, long int, __data);
 //#define __NR_sysfs            135
 
 //#define __NR_personality      136
+#ifdef __NR_personality
+#ifdef L_personality
+#include <sys/personality.h>
+_syscall1(int, personality, unsigned long int, __persona);
+#endif
+#endif
 
 //#define __NR_afs_syscall      137
 
@@ -1396,9 +1443,15 @@ weak_alias(_newselect, select);
 #endif
 
 //#define __NR_flock            143
-#ifdef L_flock
+#ifdef L___syscall_flock
 #include <sys/file.h>
-_syscall2(int,flock,int,fd, int,operation);
+#define __NR___syscall_flock __NR_flock
+static inline
+_syscall2(int, __syscall_flock, int, fd, int, operation);
+int flock(int fd, int operation)
+{
+	return(__syscall_flock(fd, operation));
+}
 #endif
 
 //#define __NR_msync            144
@@ -1447,7 +1500,15 @@ _syscall1(int, fdatasync, int, fd);
 //#define __NR__sysctl          149
 #ifdef __NR__sysctl
 #ifdef L__sysctl
-#include <linux/sysctl.h>
+struct __sysctl_args {
+	int *name;
+	int nlen;
+	void *oldval;
+	size_t *oldlenp;
+	void *newval;
+	size_t newlen;
+	unsigned long __unused[4];
+};
 _syscall1(int, _sysctl, struct __sysctl_args *, args);
 int sysctl(int *name, int nlen, void *oldval, size_t *oldlenp,
 			void *newval, size_t newlen)
@@ -1703,6 +1764,13 @@ int getresgid(gid_t *rgid, gid_t *egid, gid_t *sgid)
 #endif
 
 //#define __NR_prctl                    172
+#ifdef __NR_prctl
+#ifdef L_prctl
+#include <stdarg.h>
+//#include <sys/prctl.h>
+_syscall5(int, prctl, int, a, int, b, int, c, int, d, int, e);
+#endif
+#endif
 
 //#define __NR_rt_sigreturn             173
 //#define __NR_rt_sigaction             174
@@ -2007,42 +2075,27 @@ _syscall3(int, madvise, void*, __addr, size_t, __len, int, __advice);
 // See getdents64.c
 
 //#define __NR_fcntl64		221
-#ifdef L__fcntl64
-#ifdef __UCLIBC_HAS_LFS__
+#ifdef L___syscall_fcntl64
 #include <stdarg.h>
 #include <fcntl.h>
-#ifdef __NR_fcntl64
-#define __NR__fcntl64 __NR_fcntl64
-extern int _fcntl64(int fd, int cmd, long arg);
-
-_syscall3(int, _fcntl64, int, fd, int, cmd, long, arg);
-
-int fcntl64(int fd, int command, ...)
-{
-	long arg;
-	va_list list;
-
-	va_start(list, command);
-	arg = va_arg(list, long);
-
-	va_end(list);
-	return _fcntl64(fd, command, arg);
-}
+#ifdef __UCLIBC_HAS_LFS__
+#define __NR___syscall_fcntl64 __NR_fcntl64
+static inline
+_syscall3(int, __syscall_fcntl64, int, fd, int, cmd, long, arg);
 #else
-extern int _fcntl(int fd, int cmd, long arg);
-int fcntl64(int fd, int command, ...)
+#define __syscall_fcntl64  __syscall_fcntl
+extern int __syscall_fcntl(int fd, int cmd, ...);
+#endif
+int __libc_fcntl64(int fd, int cmd, ...)
 {
 	long arg;
 	va_list list;
-
-	va_start(list, command);
+	va_start(list, cmd);
 	arg = va_arg(list, long);
-
 	va_end(list);
-	return _fcntl(fd, command, arg);
+	return(__syscall_fcntl64(fd, cmd, arg));
 }
-#endif
-#endif
+weak_alias(__libc_fcntl64, fcntl64)
 #endif
 
 //#define __NR_security		223	/* syscall for security modules */
